(ns attendant.handler
  (:require
   [attendant.handlers.playlist :refer [playlist]]
   [attendant.handlers.witwibw :refer [witwibw]]
   [clojure.string :as string]
   [compojure.core :refer [ANY context defroutes OPTIONS routes]]
   [compojure.route :refer [not-found]]
   [ring.util.response :refer [header response]]))

(def simple-functions
  [["/witwibw" witwibw]])

(def complex-functions
  [["/playlist" playlist]])

(defroutes app
  (OPTIONS "*" [] "")
  (ANY "/" [] "attendant server")

  (->> simple-functions
       (map #(ANY (first %) [] (second %)))
       (apply routes))

  (->> complex-functions
       (map #(context (first %) [] ((second %))))
       (apply routes))

  (ANY "/directory" [] (->> (concat simple-functions complex-functions)
                            (map first)
                            (string/join "\n")
                            response))

  (not-found "nothing here (yet)"))

(defn make-handler []
  (fn [req]
    (let [body (some-> req
                       :body
                       (slurp)
                       (not-empty)
                       (read-string))
          new-req (cond-> req
                    body (update :params merge body))]
      (-> (#'app new-req)
          (header "Access-Control-Allow-Methods" "GET, DELETE, POST, PUT, OPTIONS")
          (header "Access-Control-Allow-Origin" (get-in req [:headers "origin"]))
          (header "Access-Control-Allow-Credentials" "true")
          (header "Access-Control-Allow-Headers" "Content-Type")))))
